// Copyright (c) 2012, Ladislav Thon. All rights reserved. Use of this source
// code is governed by a BSD-style license that can be found in the LICENSE file.

import 'package:darmatch/matchers.dart';

import 'package:detester/detester.dart';
part 'package:detester/test.dart';

suite() => [
  testNulls,
  testBools,
  testNumbers,
  testStrings,
  testCollections,
  testMaps,
  testExceptions,
  testTypes,
  testListDiff
];

testNulls() {
  expectThat(null, isNull());
  expectThat(0, not(isNull()));
  expectThat(0, isNotNull());
}

testBools() {
  expectThat(true, isTrue());
  expectThat(true, not(isFalse()));
  expectThat(false, isFalse());
  expectThat(false, not(isTrue()));
}

testNumbers() {
  expectThat(0, zero());
  expectThat(1, not(zero()));
  expectThat(1, positive());
  expectThat(-1, negative());
  expectThat(1, equals(1));
  expectThat(1, lessThan(2));
  expectThat(1, lessThanOrEqual(1));
  expectThat(1, lessThanOrEqual(2));
  expectThat(2, greaterThan(1));
  expectThat(2, greaterThanOrEqual(1));
  expectThat(2, greaterThanOrEqual(2));
  expectThat(1, inRange(1, 3));
  expectThat(2, inRange(1, 3));
  expectThat(3, inRange(1, 3));
  expectThat(0, not(inRange(1, 3)));
  expectThat(4, not(inRange(1, 3)));

  expectThat(0, even());
  expectThat(2, not(odd()));
  expectThat(1, odd());
  expectThat(-1, not(even()));
}

testStrings() {
  expectThat("", empty());
  expectThat("a", not(empty()));
  expectThat("a", equals("a"));
  expectThat("a", not(equals("A")));
  expectThat("a", equalsIgnoreCase("a"));
  expectThat("a", equalsIgnoreCase("A"));
  expectThat("abc", beginsWith("ab"));
  expectThat("abc", endsWith("bc"));
  expectThat("abc", contains("b"));
  expectThat("abc", matches("a.c"));
  expectThat("abc", matches(new RegExp("a.c")));
  expectThat("abcd", not(matches(@"^a.c$")));

  expectThat("abc", and([matches("a.c"), matches("a.*"), matches(".*c")]));
  expectThat("abc", or([matches("ab."), matches("a.c"), matches(".bc")]));
}

testCollections() {
  var nums = [1, 2, 3, 4];
  expectThat([], emptyCollection());
  expectThat(nums, not(emptyCollection()));
  expectThat(nums, collectionContains(1));
  expectThat(nums, not(collectionContains(0)));
  expectThat(nums, everyElement(positive()));
  expectThat(nums, someElement(even()));
  expectThat(nums, collectionLength(equals(4)));
  expectThat(nums, orderedEquals([1, 2, 3, 4]));
  expectThat(nums, unorderedEquals([4, 3, 2, 1]));
}

testMaps() {
  var map = { "a": 1, "b": 2, "c": 3 };
  expectThat({}, emptyMap());
  expectThat(map, not(emptyMap()));
  expectThat(map, mapContainsKey("a"));
  expectThat(map, mapContainsValue(1));
  expectThat(map, mapContains("a", 1));
  expectThat(map, mapLength(equals(3)));
}

class TestClass {
  returns() { }
  throws() { throw new ExpectException(null); }
}

testExceptions() {
  var f = () { };
  var g = () { throw new ExpectException(null); };
  expectThat( () => f(), returnsNormally());
  expectThat( () => f(), not(throws()));
  expectThat( () => g(), throws());
  expectThat( () => g(), not(returnsNormally()));

  var t = new TestClass();
  expectThat(t.throws, throws());
  expectThat(t.returns, returnsNormally());
}

testTypes() {
  expectThat("a", new IsType<String>());
  expectThat(1, new IsType<int>());
  expectThat(1, new IsType<num>());
  expectThat(1, not(new IsType<String>()));
  expectThat("a", not(new IsType<int>()));

  var t = new TestClass();
  expectThat(t, new IsType<TestClass>());
  expectThat(t.throws, throws(new IsType<ExpectException>()));
}

testListDiff() {
  expectThat(
      () => expectThat([1, 2, 3], orderedEquals([1, 2, 3, 4])),
      throws(caught: and([
          new IsType<ExpectException>(),
          stringForm(equals("""
1  ==  1
2  ==  2
3  ==  3
4  !=
"""))
      ]))
  );
}

